﻿Public Class ActiveJoyPad

    Inherits Windows.Forms.PictureBox

    ' ====================================================================================
    '  On the TouchScreen the MouseDown event is not fired 
    '  On the TouchScreen the MouseUp event is fired correctly
    ' ------------------------------------------------------------------------------------
    '  The touch screen event PointerDown can be used as MouseDown
    ' ------------------------------------------------------------------------------------
    '  The touch screen event WM_POINTERUPDATE and the WM_MOUSEMOVE 
    '   are mixed in the event MouseAndPointerMove
    '  The WM_POINTERUPDATE is prioritary and inhibits the following ten WM_MOUSEMOVE
    '  This prevents the incorrect values errors coming from WM_MOUSEMOVE 
    '   generated by the touch screen if you keep it pressed for a long time 
    '   and activate the simulation of the right mouse button.
    ' ====================================================================================
    Friend Event PointerDown As PointerDownEventHandler
    Friend Delegate Sub PointerDownEventHandler(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)

    Friend Event MouseAndPointerMove As MouseAndPointerMoveEventHandler
    Friend Delegate Sub MouseAndPointerMoveEventHandler(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)

    Private MouseMoveEnableCounter As Int32 = 0

    Protected Overrides Sub WndProc(ByRef msg As System.Windows.Forms.Message)
        'Const WM_POINTERCAPTURECHANGED As Integer = &H24C
        Const WM_POINTERUPDATE As Integer = &H245
        Const WM_POINTERDOWN As Integer = &H246
        Const WM_POINTERUP As Integer = &H247
        'Const WM_SETFOCUS As Integer = 7
        'Const WM_KILLFOCUS As Integer = 8
        Const WM_MOUSEMOVE As Integer = &H200
        Select Case msg.Msg
            Case WM_MOUSEMOVE
                MouseMoveEnableCounter -= 1
                If MouseMoveEnableCounter <= 0 Then
                    MouseMoveEnableCounter = 0
                    Dim pt As Point = lParamToPoint(msg.LParam)
                    Dim args As MouseEventArgs = New MouseEventArgs(Control.MouseButtons, 1, pt.X, pt.Y, 0)
                    RaiseEvent MouseAndPointerMove(Me, args)
                End If
            Case WM_POINTERUPDATE
                MouseMoveEnableCounter = 10
                Dim pt As Point = PointToClient(lParamToPoint(msg.LParam))
                Dim args As MouseEventArgs = New MouseEventArgs(MouseButtons.Left, 1, pt.X, pt.Y, 0)
                RaiseEvent MouseAndPointerMove(Me, args)
            Case WM_POINTERDOWN
                'Debug.WriteLine("DOWN")
                Dim pt As Point = PointToClient(lParamToPoint(msg.LParam))
                Dim args As MouseEventArgs = New MouseEventArgs(MouseButtons.Left, 1, pt.X, pt.Y, 0)
                RaiseEvent PointerDown(Me, args)
            Case WM_POINTERUP
                'Debug.WriteLine("UP")
        End Select
        MyBase.WndProc(msg)
    End Sub

    Private Function lParamToPoint(ByVal lParam As IntPtr) As Point
        Dim lp As UInt32 = CType(lParam, UInt32)
        Return New Point(CShort(lp And &HFFFF), CShort((lp >> 16) And &HFFFF))
    End Function

    Private mTextAlign As System.Windows.Forms.HorizontalAlignment
    Public Property TextAlign() As System.Windows.Forms.HorizontalAlignment
        Get
            Return mTextAlign
        End Get
        Set(ByVal value As System.Windows.Forms.HorizontalAlignment)
            mTextAlign = value
        End Set
    End Property

    Private Sub MyJoyPad_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        ' --------------------------------------------------------------------- Marker
        If MarkerPos.X >= 0 Then
            e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
            e.Graphics.DrawEllipse(New Pen(Color.Yellow, 3), MarkerPos.X - 12, MarkerPos.Y - 12, 25, 25)
            e.Graphics.FillEllipse(New SolidBrush(Color.LightSteelBlue), MarkerPos.X - 7, MarkerPos.Y - 7, 15, 15)
        End If
        ' --------------------------------------------------------------------- Text
        Dim flags As TextFormatFlags = TextFormatFlags.VerticalCenter Or TextFormatFlags.WordBreak
        Select Case mTextAlign
            Case HorizontalAlignment.Left : flags = flags Or TextFormatFlags.Left
            Case HorizontalAlignment.Center : flags = flags Or TextFormatFlags.HorizontalCenter
            Case HorizontalAlignment.Right : flags = flags Or TextFormatFlags.Right
        End Select
        TextRenderer.DrawText(e.Graphics, Me.Text, Me.Font, Me.ClientRectangle, Me.ForeColor, flags)
    End Sub

    Private MarkerPos As Point = New Point(-99, -99)
    Public WriteOnly Property MarkerPosition() As Point
        Set(ByVal value As Point)
            MarkerPos = value
            Me.Invalidate()
        End Set
    End Property

End Class
